@@ -4,6 +4,7 @@ import cv2
 import dask
 import matplotlib.pyplot as plt
 import numpy as np
+import rasterio
 import xarray as xr
 
 from matplotlib.animation import FuncAnimation, FFMpegWriter
@@ -27,7 +28,6 @@ class Frames(ORCBase):
         """
         super(Frames, self).__init__(xarray_obj)
 
-
     def get_piv(self, **kwargs):
         """Perform PIV computation on projected frames. Only a pipeline graph to computation is setup. Call a result to
         trigger actual computation. The dataset returned contains velocity information for each interrogation window
@@ -60,7 +60,7 @@ class Frames(ORCBase):
             kwargs["overlap"] = int(round(camera_config.window_size) / 2)
         # first get rid of coordinates that need to be recalculated
         coords_drop = list(set(self._obj.coords) - set(self._obj.dims))
-        obj = self._obj.drop(coords_drop)
+        obj = self._obj.drop_vars(coords_drop)
         # get frames and shifted frames in time
         frames1 = obj.shift(time=1)[1:]
         frames2 = obj[1:]
@@ -84,9 +84,13 @@ class Frames(ORCBase):
             lons = None
             lats = None
 
-        M = camera_config.get_M(self.h_a, reverse=True)
+        M = camera_config.get_M(self.h_a, to_bbox_grid=True, reverse=True)
         # compute row and column position of vectors in original reprojected background image col/row coordinates
-        xp, yp = helpers.xy_to_perspective(*np.meshgrid(x, np.flipud(y)), self.camera_config.resolution, M)
+        xp, yp = helpers.xy_to_perspective(
+            *np.meshgrid(x, np.flipud(y)),
+            self.camera_config.resolution,
+            M
+        )
         # ensure y coordinates start at the top in the right orientation (different from order of a CRS)
         shape_y, shape_x = self.camera_shape
         yp = shape_y - yp
@@ -140,7 +144,6 @@ class Frames(ORCBase):
         ds.velocimetry.set_encoding()
         return ds
 
-
     def project(self, resolution=None):
         """Project frames into a projected frames object, with information from the camera_config attr.
         This requires that the CameraConfig contains full gcp information. If a CRS is provided, also "lat" and "lon"
@@ -161,8 +164,15 @@ class Frames(ORCBase):
         camera_config = copy.deepcopy(self.camera_config)
         if resolution is not None:
             camera_config.resolution = resolution
-        M = camera_config.get_M(self.h_a)
+        # convert bounding box coords into row/column space
         shape = camera_config.shape
+        # get camera perspective bbox corners
+        src = camera_config.get_bbox(camera=True, h_a=self.h_a).exterior.coords[0:4]
+        dst_xy = camera_config.get_bbox().exterior.coords[0:4]
+        # get geographic coordinates bbox corners
+        dst = cv.transform_to_bbox(dst_xy, camera_config.bbox, camera_config.resolution)
+        M = cv.get_M_2D(src, dst)
+
         # prepare all coordinates
         y = np.flipud(np.linspace(
             camera_config.resolution / 2,
@@ -228,7 +238,6 @@ class Frames(ORCBase):
 
         return f
 
-
     def landmask(self, dilate_iter=10, samples=15):
         """Attempt to mask out land from water, by assuming that the time standard deviation over mean of land is much
         higher than that of water. An automatic threshold using Otsu thresholding is used to separate and a dilation
@@ -268,7 +277,6 @@ class Frames(ORCBase):
         # make mask 3-dimensional
         return (self._obj * mask)
 
-
     def normalize(self, samples=15):
         """Remove the mean of sampled frames. This is typically used to remove non-moving background from foreground, and
         helps to increase contrast when river bottoms are visible, or when the objective contains partly illuminated and
@@ -297,8 +305,7 @@ class Frames(ORCBase):
         frames_norm = ((frames_reduce - frames_min) / (frames_max - frames_min) * 255).astype("uint8")
         return frames_norm
 
-
-    def edge_detect(self, wdw_1=2, wdw_2=4):
+    def edge_detect(self, wdw_1=1, wdw_2=2):
         """Convert frames in edges, using a band convolution filter. The filter uses two slightly differently convolved
         images and computes their difference to detect edges.
 
@@ -332,7 +339,6 @@ class Frames(ORCBase):
             keep_attrs=True
         )
 
-
     def reduce_rolling(self, samples=25):
         """Remove a rolling mean from the frames (very slow, so in most cases, it is recommended to use
         Frames.normalize).
@@ -360,7 +366,6 @@ class Frames(ORCBase):
         frames_norm = frames_norm.where(roll_mean != 0, 0)
         return frames_norm
 
-
     def to_ani(
             self,
             fn,
@@ -406,7 +411,7 @@ class Frames(ORCBase):
 
         im = ax.imshow(self._obj[0], **kwargs)
         if progress_bar:
-            frames = tqdm(range(len(self._obj)))
+            frames = tqdm(range(len(self._obj)), position=0, leave=True)
         else:
             frames = range(len(self._obj))
         anim = FuncAnimation(
@@ -414,4 +419,48 @@ class Frames(ORCBase):
         )
         anim.save(fn, **video_kwargs)
 
-    plot = _frames_plot
\ No newline at end of file
+    def to_video(
+            self,
+            fn,
+            video_format=cv2.VideoWriter_fourcc(*"mp4v"),
+            fps=None
+    ):
+        """
+        Write frames to a video file without any layout
+
+        Parameters
+        ----------
+        fn : str
+            Path to output file
+        video_format : cv2.VideoWriter_fourcc, optional
+            A VideoWriter preference, default is cv2.VideoWriter_fourcc(*"mp4v")
+        fps : float, optional
+            Frames per second, if not provided, derived from original video
+
+        Returns
+        -------
+        None
+        """
+        if fps is None:
+            # estimate it from the time differences
+            fps = 1/(self._obj["time"][1].values - self._obj["time"][0].values)
+        h = self._obj.shape[1]
+        w = self._obj.shape[2]
+        out = cv2.VideoWriter(fn, video_format, fps, (w, h))
+        pbar = tqdm(self._obj, position=0, leave=True)
+        pbar.set_description("Writing frames")
+        for n, f in enumerate(pbar):
+            if len(f.shape) == 3:
+                img = cv2.cvtColor(f.values, cv2.COLOR_RGB2BGR)
+            else:
+                img = f.values
+                if n == 0:
+                    # make a scale between 0 and 255, only with first frame
+                    img_min = img.min(axis=0).min(axis=0)
+                    img_max = img.max(axis=0).max(axis=0)
+                img = np.uint8(255*((img - img_min)/(img_max - img_min)))
+                img = cv2.cvtColor(img, cv2.COLOR_GRAY2BGR)
+            out.write(img)
+        out.release()
+
+    plot = _frames_plot
